Package: interplot
Title: Plot the Effects of Variables in Interaction Terms
Version: 1.2.0
Authors@R: c(
    person("Frederick", "Solt", email = "frederick-solt@uiowa.edu", role = c("aut")),
    person("Yue", "Hu", email = "yuehu@tsinghua.edu.cn", role = c("aut", "cre")),
    person("Brenton", "Kenkel", email = "brenton.kenkel@gmail.com", role = "ctb"))
Maintainer: Yue Hu <yuehu@tsinghua.edu.cn>
Description: Plots the conditional coefficients ("marginal effects") of
    variables included in multiplicative interaction terms.
BugReports: https://github.com/sammo3182/interplot/issues
Encoding: UTF-8
Depends: R (>= 4.1.0), ggplot2
Imports: stats, abind, arm, dplyr, purrr, lme4, interactionTest
License: MIT + file LICENSE
Suggests: knitr, rmarkdown, mitools, gridExtra, merTools, brms,
        splines, testthat (>= 3.0.0)
VignetteBuilder: knitr
RoxygenNote: 8.0.0
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-06-24 09:47:09 UTC; yuehu
Author: Frederick Solt [aut],
  Yue Hu [aut, cre],
  Brenton Kenkel [ctb]
Repository: CRAN
Date/Publication: 2026-06-24 10:10:02 UTC
