# Generated by roxygen2: do not edit by hand

S3method(interplot,default)
S3method(interplot,glmerMod)
S3method(interplot,glmmi)
S3method(interplot,gmlmmi)
S3method(interplot,lmerMod)
S3method(interplot,lmmi)
S3method(interplot,mlmmi)
S3method(interplot,plot)
export(interplot)
import(dplyr)
import(ggplot2)
importFrom(abind,abind)
importFrom(arm,sim)
importFrom(dplyr,mutate)
importFrom(graphics,hist)
importFrom(interactionTest,fdrInteraction)
importFrom(purrr,map)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,plogis)
importFrom(stats,qnorm)
importFrom(stats,quantile)
