pkgdown <img src="https://user-images.githubusercontent.com/58319029/112086636-891c0600-8bc7-11eb-8ab3-9737dd1483d1.png" width = "134.435" height = "155.25" align="right" />

[![CRAN version](http://www.r-pkg.org/badges/version/interplot)](https://cran.r-project.org/package=interplot) ![](http://cranlogs.r-pkg.org/badges/grand-total/interplot)

------------------------------------------------------------------------
interplot
=========

`interplot` is a tool for plotting the conditional coefficients ("marginal effects") of variables included in multiplicative interaction terms. The function plots the changes in the coefficient of one variable in a two-way interaction term conditional on the value of the other included variable. The plot also includes simulated 95% confidential intervals of these coefficients.

To install:

* the latest released version: `install.packages("interplot")`.
* the latest developing version: `remotes::install_github("sammo3182/interplot")`.


More details are available at:

http://cran.r-project.org/web/packages/interplot/vignettes/interplot-vignette.html


Please note that this project is released with a [Contributor Code of Conduct](https://github.com/sammo3182/interplot/blob/master/CONDUCT.md). By participating in this project you agree to abide by its terms.
